/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2003      by Advanced Data Controls, Corp
 *  Copyright (C) 2004 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN  
 * 
 *  上記著作権者は，以下の (1)〜(4) の条件か，Free Software Foundation 
 *  によって公表されている GNU General Public License の Version 2 に記
 *  述されている条件を満たす場合に限り，本ソフトウェア（本ソフトウェア
 *  を改変したものを含む．以下同じ）を使用・複製・改変・再配布（以下，
 *  利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，その適用可能性も
 *  含めて，いかなる保証も行わない．また，本ソフトウェアの利用により直
 *  接的または間接的に生じたいかなる損害に関しても，その責任を負わない．
 * 
 *  @(#) $Id: rp1.c,v 1.0 2015/08/30 00:00:00 azo $
 *  
 */

/*
 * Raspberry Pi 1 内部機能用ドライバ
 */
#include <s_services.h>
#include "rp1.h"


/*
 * シリアルI/Oポート初期化ブロック
 */
const SIOPINIB siopinib_table[TNUM_SIOP] = {
    {false}
#if TNUM_SIOP >= 2
    ,
    {false}

#endif /* TNUM_SIOP >= 2 */
};


/*
 *  シリアルI/Oポート管理ブロックのエリア
 */
SIOPCB  siopcb_table[TNUM_SIOP];


/*
 *  シリアルI/OポートIDから管理ブロックを取り出すためのマクロ
 */
#define INDEX_SIOP(siopid)  ((UINT)((siopid) - 1))
#define get_siopcb(siopid)  (&(siopcb_table[INDEX_SIOP(siopid)]))


/*
 * 文字を受信したか?
 */ 
Inline BOOL
uart_getready(SIOPCB *siopcb)
{
    return(0);
}

/*
 * 文字を送信できるか?
 */
Inline BOOL
uart_putready(SIOPCB *siopcb)
{
    return(0);
}

/*
 *  受信した文字の取り出し
 */
Inline UB
uart_getchar(SIOPCB *siopcb)
{
    uint8_t c = (uint8_t)(sil_rew_mem((void*)UART0_DR) & 0xFF);	/* Read DR 0:7 */
    return(c);
}

/*
 *  送信する文字の書き込み
 */
Inline void
uart_putchar(SIOPCB *siopcb, UB c)
{
    sil_wrw_mem((void*)UART0_DR, c);		/* Write DR 0:7 */
}

/*
 *  送信割込み許可
 */
Inline void
uart_enable_send(SIOPCB *siopcb)
{
}

/*
 *  送信割込み禁止
 */
Inline void
uart_disable_send(SIOPCB *siopcb)
{
}


/*
 *  受信割込み許可
 */
Inline void
uart_enable_rcv(SIOPCB *siopcb)
{
	sil_wrw_mem((void*)UART0_IMSC, sil_rew_mem((void*)UART0_IMSC) | 0x10);	/* Receive Interrupt unmask */
}

/*
 *  受信割込み禁止
 */
Inline void
uart_disable_rcv(SIOPCB *siopcb)
{
	sil_wrw_mem((void*)UART0_IMSC, sil_rew_mem((void*)UART0_IMSC) & ~(0x10));	/* Receive Interrupt mask */
}

/*
 * カーネル起動時のログ出力用の初期化
 */
void
init_uart(void){
	sil_wrw_mem((void*)UART0_CR, 0x000);	/* Disable UART0 */
	sil_wrw_mem((void*)UART0_IBRD, 1);	/* 3000000(300MHz) / (115200 * 16) = 625/384 = 1 + 241/384 */
	sil_wrw_mem((void*)UART0_FBRD, 40);	/* 241/384 * 64 = about 40 */
	sil_wrw_mem((void*)UART0_LCRH, 0x60);	/* FIFO Disable, 8 n 1 */
	sil_wrw_mem((void*)UART0_IFLS, 0x0);	/* Receive interrupt FIFO 1/8, Transmit interrupt FIFO 1/8 */
	sil_wrw_mem((void*)UART0_IMSC, 0x0);	/* interrupt mask */
	sil_wrw_mem((void*)UART0_CR, 0x301);	/* Transmit Receive Enable UART0 */
	while((sil_rew_mem((void*)UART0_FR) & 0x10) == 0) {	/* FIFOclear */
		sil_rew_mem((void*)UART0_DR);
	}
	sil_wrw_mem((void*)UART0_ICR, 0x7FF);	/* Inerrupt clear */
}

/*
 *  オンチップのUARTからのポーリング出力
 */
void
uart_putc(char c)
{
	while((sil_rew_mem((void*)UART0_FR) & (1 << 5)) != 0);	/* FIFO space */
	sil_wrw_mem((void*)UART0_DR, c);		/* Write DR 0:7 */
}


/*
 *  SIOドライバの初期化ルーチン
 *  1ポートしかないため，あまり意味はない
 */
void
uart_initialize()
{
    SIOPCB  *siopcb;
    UINT    i;

    /*
     *  シリアルI/Oポート管理ブロックの初期化
     */
    for (siopcb = siopcb_table, i = 0; i < TNUM_SIOP; siopcb++, i++) {
        siopcb->siopinib = &(siopinib_table[i]);
        siopcb->openflag = FALSE;
        siopcb->sendflag = FALSE;
    }
}

/*
 * オープンしているポートがあるか
 */
BOOL
uart_openflag(void)
{
    return(siopcb_table[0].openflag);
}

/*
 * シリアルI/Oポートのオープン
 */
SIOPCB *
rp1_uart_opn_por(SIOPCB *p_siopcb, VP_INT exinf)
{
    p_siopcb->exinf = exinf;
    p_siopcb->getready = p_siopcb->putready = false;
    p_siopcb->openflag = true;

    return(p_siopcb);
}

/*
 * シリアルI/Oポートのオープン
 */
SIOPCB *
uart_opn_por(ID siopid, VP_INT exinf)
{
    SIOPCB  *p_siopcb = get_siopcb(siopid);
    bool_t    opnflg;
    ER      ercd;

    /*
     *  オープンしたポートがあるかをopnflgに読んでおく．
     */
    opnflg = p_siopcb->openflag;

    /*
     *  デバイス依存のオープン処理．
     */
    rp1_uart_opn_por(p_siopcb, exinf);

    /*
     *  シリアルI/O割込みのマスクを解除する．
     */
    uart_enable_rcv(p_siopcb);
    return(p_siopcb);
}


/*
 *  シリアルI/Oポートのクローズ
 */
void
uart_cls_por(SIOPCB *siopcb)
{
    /*
     *  シリアルI/O割込みをマスクする．
     */
    uart_disable_rcv(siopcb);

//    sil_wrw_mem((VP)(siopcb->siopinib->uart_control),
//        (sil_rew_mem((VP)(siopcb->siopinib->uart_control))&~UCR_UARTEN)); /* Disable Interrupt */
    
    siopcb->openflag = FALSE;
}

/*
 *  シリアルI/Oポートへの文字送信
 */
BOOL
uart_snd_chr(SIOPCB *siopcb, char c)
{
    if (uart_putready(siopcb)){
        uart_putchar(siopcb, c);
        return(TRUE);
    }
    return(FALSE);
}

/*
 *  シリアルI/Oポートからの文字受信
 */
INT
uart_rcv_chr(SIOPCB *siopcb)
{
    if (uart_getready(siopcb)) {
        return((INT)(UB) uart_getchar(siopcb));
    }
    return(-1);
}

/*
 *  シリアルI/Oポートからのコールバックの許可
 */
void
uart_ena_cbr(SIOPCB *siopcb, UINT cbrtn)
{
    switch (cbrtn) {
        case SIO_ERDY_SND:
            uart_enable_send(siopcb);
            break;
        case SIO_ERDY_RCV:
            uart_enable_rcv(siopcb);
            break;
    }
}


/*
 *  シリアルI/Oポートからのコールバックの禁止
 */
void
uart_dis_cbr(SIOPCB *siopcb, UINT cbrtn)
{
    switch (cbrtn) {
        case SIO_ERDY_SND:
            uart_disable_send(siopcb);
            break;
        case SIO_ERDY_RCV:
            uart_disable_rcv(siopcb);
            break;
    }
}



/*
 *  シリアルI/Oポートに対する割込み処理
 */
static void
uart_isr_siop(SIOPCB *siopcb)
{
    if (uart_getready(siopcb)) {
        /*
         *  受信通知コールバックルーチンを呼び出す．
         */
        uart_ierdy_rcv(siopcb->exinf);
    }
    if (uart_putready(siopcb)) {
        /*
         *  送信可能コールバックルーチンを呼び出す．
         */
        uart_ierdy_snd(siopcb->exinf);
    }
}



/*
 *  SIOの割込みサービスルーチン
 */
void
uart_isr0()
{
    SIOPCB *p_siopcb = &(siopcb_table[0]);
	uint_t status = sil_rew_mem((void*)UART0_MIS);

	if((status & (1 << 4)) != 0) {	/* Receive masked interrupt status. */
	        /*
	         *  受信通知コールバックルーチンを呼び出す．
	         */
                uart_ierdy_rcv(p_siopcb->exinf);
		sil_wrw_mem((void*)UART0_ICR, (1 << 4));	/* Receive interrupt clear. */
	}
	if((status & (1 << 5)) != 0) {	/* Transmit masked interrupt status. */
	        /*
	         *  送信可能コールバックルーチンを呼び出す．
	         */
                uart_ierdy_snd(p_siopcb->exinf);
		sil_wrw_mem((void*)UART0_ICR, (1 << 5));	/* Transmit interrupt clear. */
	}
	if((status & (1 << 6)) != 0) {	/* Receive timeout masked interrupt status. */
		sil_wrw_mem((void*)UART0_ICR, (1 << 6));	/* Receive timeout interrupt clear. */
	}
	if((status & (1 << 7)) != 0) {	/* Framing error masked interrupt status. */
		sil_wrw_mem((void*)UART0_ICR, (1 << 7));	/* Framing error interrupt clear. */
	}
	if((status & (1 << 8)) != 0) {	/* Parity error masked interrupt status. */
		sil_wrw_mem((void*)UART0_ICR, (1 << 8));	/* Parity error interrupt clear. */
	}
	if((status & (1 << 9)) != 0) {	/* Break error masked interrupt status. */
		sil_wrw_mem((void*)UART0_ICR, (1 << 9));	/* Break error interrupt clear. */
	}
	if((status & (1 << 10)) != 0) {	/* Overrun error masked interrupt status. */
		sil_wrw_mem((void*)UART0_ICR, (1 << 10));	/* Overrun error interrupt clear. */
	}
}

#if TNUM_SIOP >= 2
void
uart_isr1()
{
    uart_isr_siop(&(siopcb_table[1]));
}
#endif
