/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2003      by Advanced Data Controls, Corp
 *  Copyright (C) 2004 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  上記著作権者は，以下の (1)〜(4) の条件か，Free Software Foundation 
 *  によって公表されている GNU General Public License の Version 2 に記
 *  述されている条件を満たす場合に限り，本ソフトウェア（本ソフトウェア
 *  を改変したものを含む．以下同じ）を使用・複製・改変・再配布（以下，
 *  利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，その適用可能性も
 *  含めて，いかなる保証も行わない．また，本ソフトウェアの利用により直
 *  接的または間接的に生じたいかなる損害に関しても，その責任を負わない．
 * 
 *  @(#) $Id: rp1.h 1838 2015-08-25 00:00:00Z azo $
 */

#ifndef _RP1_H_
#define _RP1_H_

#ifndef _MACRO_ONLY
#include <itron.h>
#include <sil.h>
#endif /* _MACRO_ONLY */

#include "armv6.h"
#include "sys_config.h"
#include "cpu_config.h"




/*
 * 割込みハンドラ番号から，IRC操作のためのビットパターンを求めるマクロ(0<-intno<-31)
 */
#define INTNO_BITPAT(intno) (1U << intno)

/*
 *  割込みコントローラ関係
 */
#define AIC_BASE		(0x2000B200)
#define AIC_BASIC		(AIC_BASE + 0x00)
#define AIC_PEND1		(AIC_BASE + 0x04)
#define AIC_PEND2		(AIC_BASE + 0x08)
#define AIC_FIQ_CONTROL		(AIC_BASE + 0x0C)
#define AIC_ENABLE_IRQ1		(AIC_BASE + 0x10)
#define AIC_ENABLE_IRQ2		(AIC_BASE + 0x14)
#define AIC_ENABLE_BASIC	(AIC_BASE + 0x18)
#define AIC_DISABLE_IRQ1	(AIC_BASE + 0x1C)
#define AIC_DISABLE_IRQ2	(AIC_BASE + 0x20)
#define AIC_DISABLE_BASIC	(AIC_BASE + 0x24)

#define IRQ_ARMTIMER	0
#define IRQ_UART0	89

/*
 *  タイマ関連
 */
#define ARM_TIMER_BASE	(0x2000B400)
#define ARM_TIMER_LOD	(ARM_TIMER_BASE + 0x00)
#define ARM_TIMER_VAL	(ARM_TIMER_BASE + 0x04)
#define ARM_TIMER_CTL	(ARM_TIMER_BASE + 0x08)
#define ARM_TIMER_CLI	(ARM_TIMER_BASE + 0x0C)
#define ARM_TIMER_RIS	(ARM_TIMER_BASE + 0x10)
#define ARM_TIMER_MIS	(ARM_TIMER_BASE + 0x14)
#define ARM_TIMER_RLD	(ARM_TIMER_BASE + 0x18)
#define ARM_TIMER_DIV	(ARM_TIMER_BASE + 0x1C)
#define ARM_TIMER_CNT	(ARM_TIMER_BASE + 0x20)

/*
 *  UART関連
 */
#define GPFSEL0 0x20200000
#define GPFSEL1 0x20200004
#define GPFSEL2 0x20200008
#define GPSET0  0x2020001C
#define GPCLR0  0x20200028
#define GPPUD       0x20200094
#define GPPUDCLK0   0x20200098

#define UART0_BASE	(0x20201000)
#define UART0_DR	(UART0_BASE + 0x00)	/* Data Register */
#define UART0_RSRECR	(UART0_BASE + 0x04)
#define UART0_FR	(UART0_BASE + 0x18)	/* Flag register */
#define UART0_ILPR	(UART0_BASE + 0x20)
#define UART0_IBRD	(UART0_BASE + 0x24)	/* Integer Baud rate divisor */
#define UART0_FBRD	(UART0_BASE + 0x28)	/* Fractional Baud rate divisor	*/
#define UART0_LCRH	(UART0_BASE + 0x2C)	/* Line Control register */
#define UART0_CR	(UART0_BASE + 0x30)	/* Control register */
#define UART0_IFLS	(UART0_BASE + 0x34)	/* Interupt FIFO Level Select Register */
#define UART0_IMSC	(UART0_BASE + 0x38)	/* Interupt Mask Set Clear Register */
#define UART0_RIS	(UART0_BASE + 0x3C)	/* Raw Interupt Status Register */
#define UART0_MIS	(UART0_BASE + 0x40)	/* Masked Interupt Status Register */
#define UART0_ICR	(UART0_BASE + 0x44)	/* Interupt Clear Register */
#define UART0_DMACR	(UART0_BASE + 0x48)	/* DMA Control Register */
#define UART0_ITCR	(UART0_BASE + 0x80)	/* Test Control register */
#define UART0_ITIP	(UART0_BASE + 0x84)	/* Integration test input reg */
#define UART0_ITOP	(UART0_BASE + 0x88)	/* Integration test output reg */
#define UART0_TDR	(UART0_BASE + 0x8C)	/* Test Data reg */

#ifndef _MACRO_ONLY

/*
 * IRC操作関数
 */

/*
 * 割込み要求のマスク
 */
Inline void
rp1_disable_int(uint32_t no, uint32_t mask)
{
	switch(no) {
	case 0: 
		sil_wrw_mem((void *)(AIC_DISABLE_BASIC), mask);
		break;
	case 1: 
		sil_wrw_mem((void *)(AIC_DISABLE_IRQ1), mask);
		break;
	case 2: 
		sil_wrw_mem((void *)(AIC_DISABLE_IRQ2), mask);
		break;
	}
}

/*
 * 割込み要求のマスクの解除
 */
Inline void
rp1_enable_int(uint32_t no, uint32_t mask)
{
	switch(no) {
	case 0: 
		sil_wrw_mem((void *)(AIC_ENABLE_BASIC), mask);
		break;
	case 1: 
		sil_wrw_mem((void *)(AIC_ENABLE_IRQ1), mask);
		break;
	case 2: 
		sil_wrw_mem((void *)(AIC_ENABLE_IRQ2), mask);
		break;
	}
}

/*
 * 割込み要求のクリア
 */
Inline void
rp1_clear_int(uint32_t no, uint32_t mask)
{
}

/*
 * 割込み要求のチェック
 */
Inline bool_t
rp1_probe_int(uint32_t no, uint32_t mask)
{
	switch(no) {
	case 0: 
		return((sil_rew_mem((void *)(AIC_BASIC)) & mask) == mask);
	case 1: 
		return((sil_rew_mem((void *)(AIC_PEND1)) & mask) == mask);
	case 2: 
		return((sil_rew_mem((void *)(AIC_PEND2)) & mask) == mask);
	}
}

/*
 * 終了処理
 */
Inline void
rp1_exit(void)
{    
}

/* 
 *  サイクルカウンタの読み込み 
 */ 
Inline uint32_t 
cycle_counter_get_current(void)
{ 
        return sil_rew_mem((void*)(ARM_TIMER_CNT)); 
}

/*
 *  トレースログに関する定義
 *  サイクルカウンタを用いる．
 */
#define TRACE_GET_TIM()   cycle_counter_get_current()

/*
 * 内蔵UART用 簡易SIOドライバ
 */
/*
 * カーネル起動時用の初期化(sys_putcを使用するため)
 */
extern void init_uart(void);


/*
 *  シリアルI/Oポート初期化ブロック
 */
typedef struct sio_port_initialization_block 
{
	bool_t open;
}
SIOPINIB;

/*
 *  シリアルI/Oポート管理ブロックの定義
 */
typedef struct sio_port_control_block 
{
    const SIOPINIB  *siopinib;  /* シリアルI/Oポート初期化ブロック */
    VP_INT          exinf;      /* 拡張情報 */
    BOOL    openflag;       /* オープン済みフラグ */
    BOOL    sendflag;       /* 送信割込みイネーブルフラグ */
    BOOL    getready;       /* 文字を受信した状態 */
    BOOL    putready;       /* 文字を送信できる状態 */

}SIOPCB;


/*
 *  コールバックルーチンの識別番号
 */
#define SIO_ERDY_SND    1u      /* 送信可能コールバック */
#define SIO_ERDY_RCV    2u      /* 受信通知コールバック */

/*
 *  オンチップのUARTからのポーリング出力
 */
extern void uart_putc(char c);

/*
 *  SIOドライバの初期化ルーチン
 */
extern void uart_initialize(void);

/*
 *  オープンしているポートがあるか？
 */
extern BOOL uart_openflag(void);

/*
 *  シリアルI/Oポートのオープン
 */
extern SIOPCB   *uart_opn_por(ID siopid, VP_INT exinf);

/*
 *  シリアルI/Oポートのクローズ
 */
extern void uart_cls_por(SIOPCB *siopcb);

/*
 *  シリアルI/Oポートへの文字送信
 */
extern BOOL uart_snd_chr(SIOPCB *siopcb, char c);

/*
 *  シリアルI/Oポートからの文字受信
 */
extern INT  uart_rcv_chr(SIOPCB *siopcb);

/*
 *  シリアルI/Oポートからのコールバックの許可
 */
extern void uart_ena_cbr(SIOPCB *siopcb, UINT cbrtn);

/*
 *  シリアルI/Oポートからのコールバックの禁止
 */
extern void uart_dis_cbr(SIOPCB *siopcb, UINT cbrtn);

/*
 *  SIOの割込みサービスルーチン
 */
extern void uart_in_isr(void);
extern void uart_out_isr(void);

/*
 *  シリアルI/Oポートからの送信可能コールバック
 */
extern void uart_ierdy_snd(VP_INT exinf);

/*
 *  シリアルI/Oポートからの受信通知コールバック
 */
extern void uart_ierdy_rcv(VP_INT exinf);



#endif /* _MACRO_ONLY */
#endif /* _RP1_H_ */

